@@ -1,6 +1,8 @@
 import abc
+import base64
 from dataclasses import dataclass
 from datetime import date, datetime, timedelta, timezone
+import io
 from json import dumps
 import time
 import asyncio
@@ -10,14 +12,16 @@ import pytz
 import httpx
 import gidgethub.httpx
 import matplotlib.pyplot as plt
-from bokeh.plotting import figure, output_file, show, save
-from bokeh.embed import components
-from bokeh.models import HoverTool, Range1d, Title
-from bokeh.io import output_notebook
+#from bokeh.io import export_png
+#from bokeh.plotting import figure, output_file, show, save
+#from bokeh.embed import components
+#from bokeh.models import HoverTool, Range1d, Title
+#from bokeh.io import output_notebook
 import pandas as pd
+import seaborn as sns
 
 
-plt.style.use('bmh')
+#plt.style.use('bmh')
 
 
 @dataclass
@@ -370,22 +374,39 @@ def plot_line(data, title:str, x_title:str, y_title:str, x_axis_type=None, width
     if x_axis_type == "linear":
         x_range = [str(v) for v in x]
         
-    p = figure(tools="save", background_fill_color="#efefef", #x_range=x_range, 
-               x_axis_type=x_axis_type, toolbar_location="below")
-    p.line(x=x, y=y, color="navy")
-    p.xgrid.grid_line_color = None
-    p.ygrid.grid_line_color = "white"
-    p.grid.grid_line_width = 2
-    p.xaxis.major_label_text_font_size="12pt"
-    p.add_layout(Title(text=title, align="center"), "above")
-    p.add_layout(Title(text=x_title, align="center"), "below")
-    p.add_layout(Title(text=y_title, align="center"), "left")   
-    p.y_range = Range1d(0, int(max_y * 1.2 + 1))
-    return p
+    # Set Seaborn style
+    sns.set_theme(style="whitegrid")
+
+    # Create the plot
+    fig, ax = plt.subplots()
+
+    # Set background color
+    fig.set_facecolor('#efefef')
+    ax.set_facecolor('#efefef')
+
+    # Plot the line
+    ax.plot(x, y, color="navy")
+
+    # Customize grid lines
+    ax.grid(True, which='both', linewidth=2)
+    ax.xaxis.grid(False)  # Remove x-axis grid lines
+    ax.yaxis.grid(True, color='white')  # Set y-axis grid lines to white
+
+    # Set axis labels and title
+    ax.set_title(title, fontsize=16, pad=20)
+    ax.set_xlabel(x_title, fontsize=12, labelpad=15)
+    ax.set_ylabel(y_title, fontsize=12, labelpad=15)
+
+    # Set y-axis range
+    ax.set_ylim(0, int(max_y * 1.2 + 1))
+
+    # Adjust font size for x-axis labels
+    ax.tick_params(axis='x', labelsize=12)    
     
     
 def plot_bug_rate(start:datetime, end:datetime, issues:list[Issue], who:str,
-                  must_include_labels:list[str], must_exclude_labels:list[str]|None=None, interval=7):
+                  must_include_labels:list[str], must_exclude_labels:list[str]|None=None, interval=7,
+                  as_md: bool = False) -> str:
     counts = []
     dates = []
     counts = {}
@@ -397,8 +418,24 @@ def plot_bug_rate(start:datetime, end:datetime, issues:list[Issue], who:str,
         counts[start] = count
         start += timedelta(days=interval)
         last = count
-    plot = plot_line(counts, f"Open bug count for {who}", "Date", "Count", x_axis_type="datetime", width=7)
-    return components(plot)
+    plot_line(counts, f"Open bug count for {who}", "Date", "Count", x_axis_type="datetime", width=7)
+    if True:
+        # Ideally we would follow this path and render the plot with inline
+        # data, but this doesn't work in GitHub markdown preview.
+        # Save the plot to an in-memory buffer
+        buf = io.BytesIO()
+        plt.savefig(buf, format='png')
+        buf.seek(0)
+        # Encode the image to base64
+        img_base64 = base64.b64encode(buf.read()).decode('utf-8')
+        return f"![](data:image/png;base64,{img_base64})" if as_md else \
+                f'<img src="data:image/png;base64,{img_base64}">'
+    else:
+        # Instead we have to save the plot to file and link to it. But we don't want
+        # lots of files lying around so we just use the same name and hope for the best.
+        plt.savefig("bugcount.png")
+        return '![](bugcount.png)' if as_md else '<img src="bugcount.png">'
+    return img_md
 
 
 class FormatterABC(abc.ABC):
@@ -529,7 +566,7 @@ class MarkdownFormatter(FormatterABC):
         return '\n'
 
     def line_separator(self) -> str:
-        return '\n\n'
+        return '\n' if self.as_table else '\n\n'
             
 
 def get_subset(issues:list[Issue], members: set[str], bug_flag: bool, bug_label: str = 'bug') -> Generator[Issue, None, None]:
@@ -767,7 +804,7 @@ def get_training(org: str, repo: str, token: str, out: str|None=None, verbose: b
     output_result(out, result, now)
 
 
-def find_top_terms(issues:list[Issue], formatter: FormatterABC, min_count:int=3):
+def find_top_terms(issues:list[Issue], formatter: FormatterABC, min_count:int=5):
     """
     Find the most common terms in the issue titles. First we remove common words and then
     count the remaining words. We then sort them by count.
@@ -777,7 +814,14 @@ def find_top_terms(issues:list[Issue], formatter: FormatterABC, min_count:int=3)
                   'when', 'how', 'why', 'what', 'which', 'who', 'whom', 'whose', 'i', 'you', 'he', 'she', 
                   'it', 'we', 'they', 'me', 'him', 'her', 'us', 'them', 'my', 'mine', 'your', 'yours', 
                   'his', 'her', 'hers', 'its', 'our', 'ours', 'their', 'theirs', 'myself', 'yourself', 
-                  'himself', 'herself', 'itself', 'ourselves', 'yourselves', 'themselves', 'all']
+                  'himself', 'herself', 'itself', 'ourselves', 'yourselves', 'themselves', 'all',
+                  'cannot', 'without', 'name', 'vs', 'pylance', 'python', 'show', 'add', 'support',
+                  'not', 'after', 'does', 'no', 'working', 'doesn\'t', 'can\'t', 'won\'t', 'shouldn\'t',
+                  'unable', 'visual', 'studio', 'up', 'if', 'only', 'microsoft', 'using', '-',
+                  'work', 'should', 'vscode', 'don\'t', 'offer', 'over', 'incorrect', 'inside',
+                  'being', 'could', 'go', 'showing', 'have', 'shown', 'even', 'has', 'instead',
+                  'recognized', 'issue', 'new', 'allow', 'fails', 'out', 'long', 'available', 
+                  'problem', 'get', 'until', 'can', 'like', 'debugpy']
     issues_with_term = {}
     for issue in issues:
         title = issue.title.lower()
@@ -794,20 +838,21 @@ def find_top_terms(issues:list[Issue], formatter: FormatterABC, min_count:int=3)
     # Sort issues_with_term by length of list descending
     sorted_terms = sorted(issues_with_term.items(), key=lambda x: len(x[1]), reverse=True)
     report_sections = []
+    now = datetime.now()
     for term, issues in sorted_terms:
         if len(issues) < min_count:
             break
 
         report_sections.append(
             formatter.heading(3, f"Issues with term '{term}'") + \
-            formatter.line_separator() + \
-            formatter.line_separator().join([(formatter.line(False, '', i)+'\n\n') for i in issues]) + \
+            ''.join([(formatter.line(False, '', i, \
+                    op=date_diff(now, i.created_at).days)) for i in issues]) + \
             formatter.line_separator()
         )
 
         issues_with_term[term] = len(issues)
 
-    return (formatter.line_separator() * 2).join(report_sections)
+    return (formatter.line_separator() * 3).join(report_sections)
 
 
 def create_report(org: str, repo: str, token: str, out: str|None=None, 
@@ -828,16 +873,22 @@ def create_report(org: str, repo: str, token: str, out: str|None=None,
                            formatter=formatter, days=days, stale=stale, show_all=show_all)
     
     termranks = find_top_terms(issues, formatter)
-    if fmt == '.html':
-        script, chartdiv = plot_bug_rate(now-timedelta(days=xrange), now, issues,
-                               repo, [bug_label], interval=1)
-        result = f"""<!DOCTYPE html>
+    if fmt == '.txt':
+        result = report + '\n\n' + termranks
+    else:
+        if show_all:
+            chart = plot_bug_rate(now-timedelta(days=xrange), now, issues,
+                               repo, [bug_label], interval=1, as_md=(fmt!='.md'))
+        else:
+            chart = ''
+        if fmt == '.md':
+            result = report + '\n\n' + termranks + '\n\n' + chart
+        else:
+            result = f"""<!DOCTYPE html>
 <html lang="en">
     <head>
         <meta charset="utf-8">
         <title>Repo report for {org}/{repo} on {format_date(now)}</title>
-        <script src="https://cdn.bokeh.org/bokeh/release/bokeh-2.4.2.min.js"></script>
-        {script}
     </head>
     <body>
     {report}
@@ -846,11 +897,12 @@ def create_report(org: str, repo: str, token: str, out: str|None=None,
     {termranks}
     <br>
     <br>    
-    {chartdiv}
+    {chart}
     </body>
 </html>"""
-    else:
-        result = report + '\n\n' + termranks
 
     output_result(out, result, now)
 
+
+
+     
